<?php

//
// Copyright (C) 2013, 2014 FromDual GmbH
//
// This program is commercial software. Use of this software is governed
// by your applicable license agreement with FromDual GmbH
//

/*

	openRepositoryDatabase()
	closeRepositoryDatabase($dbh)
	upgradeRepositoryDatabase()
	createTableCluster($dbh)
	createTableVip($dbh)
	createTableInterface($dbh)
	createTableServer($dbh)
	createRepositoryDatabase()
	createTableUser($dbh)
	setEncryptedLicense($dbh, $pLicense)
	getEncryptedLicense($dbh)
	getMocIdentifier($dbh)
	decryptLicense($pKey, $pEncryptedLicense)
	readEncryptLicenseFromFile($pFile)
	generateMocIdentifier()
	createTableMocIdentifier($dbh)

*/

// ---------------------------------------------------------------------
function openRepositoryDatabase()
// ---------------------------------------------------------------------
{
  global $gRepository;

  $rc = OK;

  $user     = null;
  $password = null;
  $dsn      = 'sqlite:' . $gRepository;

  $dbh = null;
  try {
    $dbh = new PDO($dsn, $user, $password);
  }
  catch (PDOException $e) {
    $rc = 803;

    print '<p>Connection failed: ' . $e->getMessage() . " (rc=$rc).<br />\n";
    print "Make sure user has access to $gRepository</p>\n";
    return array($rc, $dbh);
  }

  $dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
  $dbh->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);

  return array($rc, $dbh);
}

// ---------------------------------------------------------------------
function closeRepositoryDatabase($dbh)
// ---------------------------------------------------------------------
{
  $dbh = null;
}

// ---------------------------------------------------------------------
function upgradeRepositoryDatabase()
// ---------------------------------------------------------------------
{
}

// ---------------------------------------------------------------------
function createTableCluster($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;

  // Type:
  // 1: Master/Slave
  // 2: Master/Master
  // 3: Galera
  // 4: a/p Failover Cluster
  // 5: MySQL Cluster

  $sql = "CREATE TABLE IF NOT EXISTS clusters (
  cluster_id     INTEGER
, name           TEXT
, last_change_ts INTEGER
, type           INTEGER
, PRIMARY KEY (cluster_id)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 900;
    print "<pre>"; print_r($e); print "</pre>\n";
    print "<p>(rc=$rc)</p>\n";
  }

  $sql = "CREATE UNIQUE INDEX cluster_name ON clusters (name)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 901;
    print "<pre>"; print_r($e); print "</pre>\n";
    print "<p>(rc=$rc)</p>\n";
  }

  return $rc;
}

// ---------------------------------------------------------------------
function createTableVip($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;

  // ipaddr_type:
  // 1: ifconfig
  // 2: ip addr show

  $sql = "CREATE TABLE IF NOT EXISTS vips (
  vip_id      INTEGER
, ip_address  TEXT
, name        TEXT
, ipaddr_type INTEGER
, interface   TEXT
, alias       INTEGER
, primary_id  INTEGER
, failover_id INTEGER
, location_id INTEGER
, cluster_id  INTEGER
, PRIMARY KEY (vip_id)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 902;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  $sql = "CREATE UNIQUE INDEX ip_address ON vips (ip_address)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 903;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  return $rc;
}

// ---------------------------------------------------------------------
function createTableInterface($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;

  $sql = "CREATE TABLE IF NOT EXISTS interfaces (
  interface_id   INTEGER
, name           TEXT
, server_id      INTEGER
, PRIMARY KEY (interface_id)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 929;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  return $rc;
}

// ---------------------------------------------------------------------
function createTableServer($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;

  $sql = "CREATE TABLE IF NOT EXISTS servers (
  server_id      INTEGER
, name           TEXT
, default_ip     TEXT
, os_user        TEXT
, cluster_id     INTEGER
, PRIMARY KEY (server_id)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 928;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }


  $sql = "CREATE INDEX server_name ON servers (name)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 821;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  return $rc;
}

// ---------------------------------------------------------------------
function createRepositoryDatabase()
// ---------------------------------------------------------------------
{
  $rc = OK;

  list($ret, $dbh) = openRepositoryDatabase();
  $rc += $ret;

  if ( $ret != 0 ) {
    return $ret;
  }

  // Table for version history

  $sql = "CREATE TABLE IF NOT EXISTS versions (
  version    TEXT PRIMARY KEY
, mr_version INTEGER
, timestamp  INTEGER
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 804;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  $sql = "INSERT INTO versions (version, mr_version, timestamp) VALUES (:version, :mr_version, :timestamp)";
  $stmt = $dbh->prepare($sql);

  $version    = 'v1.0';
  $mr_version = 100;
  $timestamp  = mktime();

  $stmt->bindParam(':version', $version, PDO::PARAM_STR);
  $stmt->bindParam(':mr_version', $mr_version, PDO::PARAM_INT);
  $stmt->bindParam(':timestamp', $timestampe, PDO::PARAM_INT);
  try {
    $stmt->execute();
  }
  catch (PDOException $e) {
    $rc = 805;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }
  $stmt->closeCursor();


  // Table for license

  $sql = "CREATE TABLE IF NOT EXISTS licenses (
  license    TEXT
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 806;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }


  // Table for nodes

  // todo: name should be UNIQUE
  // add index on, INDEX (name)
  // create unique index name on node (name);

  $sql = "CREATE TABLE IF NOT EXISTS nodes (
  node_id                INTEGER
, name                   TEXT
, last_change_ts         INTEGER
, status                 TEXT
, node_type              TEXT
, hostname               TEXT
, basedir                TEXT
, datadir                TEXT
, my_cnf                 TEXT
, port                   INTEGER
, database_user          TEXT
, database_user_password TEXT
, error_log              TEXT
, pid_file               TEXT
, read_only              INTEGER
, server_id              INTEGER
, role_id                INTEGER
, cluster_id             INTEGER
, PRIMARY KEY (node_id)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 807;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  // Table for last database checks

  // lc  = last check
  // lsc = last successful check

  $sql = "CREATE TABLE IF NOT EXISTS checks (
  node_id                      INTEGER
, type                         TEXT
, name                         TEXT
, last_check_ts                INTEGER
, last_check_status            TEXT
, last_successful_check_ts     INTEGER
, last_successful_check_status TEXT
, PRIMARY KEY (node_id, name)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 802;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  $sql = "CREATE INDEX name_dc ON checks (name)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 810;
    print "<p>" . $e->getMessage() . " (rc=$rc)</p>\n";
  }

  $ret = createTableCluster($dbh);
  if ( $ret != 0 ) {
    $rc = 814;
    print "<p>(rc=$rc)</p>\n";
  }

  // $rc = createTableClusterDatabase($dbh);

  $ret = createTableVip($dbh);
  if ( $ret != 0 ) {
    $rc = 811;
    print "<p>(rc=$rc)</p>\n";
  }

  //$rc = createTableClusterVip($dbh);

  $ret = createTableServer($dbh);
  if ( $ret != OK ) {
    $rc = 812;
    print "<p>(rc=$rc)</p>\n";
  }

  $ret = createTableUser($dbh);
  if ( $ret != OK ) {
    $rc = 813;
    print "<p>Creating table users failed (rc=$rc)</p>\n";
  }

  $ret = createTableMocIdentifier($dbh);
  if ( $ret != OK ) {
    $rc = 816;
    print "<p>Creating table moc_identifier failed (rc=$rc)</p>\n";
  }


  // generate MOC identifier

  $moc_identifier = generateMocIdentifier();

  $sql = "INSERT INTO moc_identifier (moc_identifier) VALUES (:moc_identifier)";
  $stmt = $dbh->prepare($sql);
  $stmt->bindValue(':moc_identifier', $moc_identifier, PDO::PARAM_STR);

  try {
    $stmt->execute();
  }
  catch (PDOException $e) {
    $rc = 875;
  }

  closeRepositoryDatabase($dbh);
  return $rc;
}

// ---------------------------------------------------------------------
function createTableUser($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;

  $sql = "CREATE TABLE IF NOT EXISTS users (
  user_id        INTEGER
, login_name     TEXT
, password_hash  TEXT
, email_address  TEXT
, first_name     TEXT
, last_name      TEXT
, mobile         TEXT
, role_id        INTEGER
, PRIMARY KEY (user_id)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 892;
    return $rc;
  }

  $sql = "CREATE UNIQUE INDEX login_name ON users (login_name)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 893;
    return $rc;
  }

  $sql = "INSERT INTO users (login_name, password_hash)
VALUES (:login_name, :password_hash)";

  $stmt = $dbh->prepare($sql);

  $user = 'admin';
  $hash = md5('admin');

  $stmt->bindValue(':login_name', $user, PDO::PARAM_STR);
  $stmt->bindValue(':password_hash', $hash, PDO::PARAM_STR);

  try {
    $stmt->execute();
  }
  catch (PDOException $e) {
    $rc = 888;
    // print_r($e);
    return $rc;
  }

  return $rc;
}

// ---------------------------------------------------------------------
function setEncryptedLicense($dbh, $pLicense)
// ---------------------------------------------------------------------
{
  $rc = OK;
  $cnt = 0;

  $sql = "UPDATE licenses SET license = :license WHERE 1";
  $stmt = $dbh->prepare($sql);
  $stmt->bindValue(':license', $pLicense);
  try {
    $stmt->execute();
    $cnt = $stmt->rowCount();
  }
  catch (PDOException $e) {
    $rc = 994;
    $cnt = $e->getMessage();
  }
  $stmt->closeCursor();

  // We do not have a license yet
  if ( $cnt == 0 ) {

    $sql = "INSERT INTO licenses VALUES (:license)";
    $stmt = $dbh->prepare($sql);
    $stmt->bindValue(':license', $pLicense);
    try {
      $stmt->execute();
      $cnt = $stmt->rowCount();
    }
    catch (PDOException $e) {
      $rc = 993;
      $cnt = $e->getMessage();
    }
    $stmt->closeCursor();
  }

  return array($rc, $cnt);
}

// ---------------------------------------------------------------------
function getEncryptedLicense($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;
  $lLicense = '';

  $sql = "SELECT * FROM licenses";
  // This seems to be the first part of the code
  // where we access the database and check if
  // it is working...
  try {
    $stmt = $dbh->prepare($sql);
    $stmt->execute();
  }
  catch (PDOException $e) {
    $rc = 992;
    $msg = $e->getMessage();
    return array($rc, $msg);
  }

  if ( $result = $stmt->fetch(PDO::FETCH_ASSOC) ) {
    $lLicense = $result['license'];
  }
  $stmt->closeCursor();

  if ( $lLicense == '' ) {
    $rc = 991;
    $msg = 'No License found. Please get a new License.';
    return array($rc, $msg);
  }

  return array($rc, $lLicense);
}

// ---------------------------------------------------------------------
function getMocIdentifier($dbh)
// ---------------------------------------------------------------------
{

  $rc = OK;
  $lMocIndentifier = '';

  $sql = "SELECT * FROM moc_identifier";

  try {
    $stmt = $dbh->prepare($sql);
    $stmt->execute();
  }
  catch (PDOException $e) {
    $rc = 998;
    $msg = $e->getMessage();
		logMessage(LOG_ERR, $msg . " (rc=$rc)");
    return $lMocIndentifier;
  }

  if ( $result = $stmt->fetch(PDO::FETCH_ASSOC) ) {
    $lMocIndentifier = $result['moc_identifier'];
  }
  $stmt->closeCursor();

  if ( $lMocIndentifier == '' ) {
    $rc = 988;
    $msg = 'No License found. Please get a new License.';
		logMessage(LOG_ERR, $msg . " (rc=$rc)");
    return $lMocIndentifier;
  }

  return $lMocIndentifier;
}

// ---------------------------------------------------------------------
function decryptLicense($pKey, $pEncryptedLicense)
// redundant code with admin/lib/encryptLicense.inc
// ---------------------------------------------------------------------
{
  $rc = OK;

  $lDecryptedLicense = '';

  for ( $i = 0; $i < strlen($pEncryptedLicense); $i = $i + 2 ) {
    $lDecryptedLicense .= chr(hexdec($pEncryptedLicense[$i] . $pEncryptedLicense[$i+1]) ^ $pKey[13]);
  }

  $aFields = array();
  foreach ( explode(';', $lDecryptedLicense) as $key => $value ) {
    switch ( $key ) {
		case 0:
			$aFields['company'] = $value;
			break;
		case 1:
			$aFields['type'] = $value;
			break;
		case 2:
			$aFields['mysql_nodes'] = $value;
			break;
		case 3:
			$aFields['master_nodes'] = $value;
			break;
		case 4:
			$aFields['slave_nodes'] = $value;
			break;
		case 5:
			$aFields['galera_nodes'] = $value;
			break;
		case 6:
			$aFields['expiry_date'] = $value;
			break;
		case 7:
			$aFields['moc_key'] = $value;
			break;
		default:
    }
  }

	return $aFields;
}

// ---------------------------------------------------------------------
function readEncryptLicenseFromFile($pFile)
// ---------------------------------------------------------------------
{
  $rc = OK;
  $lEncryptedLicense = '';

  $handle = @fopen($pFile, 'r');
  if ( $handle === false ) {
    $rc = 990;
    $err = error_get_last();
    $msg = "Failed opening file: Error was '" . $err['message'] . "'";
    return array($rc, $msg);
  }

  while ( ($buffer = fgets($handle, 256)) !== false ) {
    $lEncryptedLicense = trim($buffer);
  }
  if ( ! feof($handle) ) {
    $rc = 989;
    $msg = "Error: unexpected fgets() fail.";
    return array($rc, $msg);
  }
  fclose($handle);

  return array($rc, $lEncryptedLicense);
}

// ---------------------------------------------------------------------
function generateMocIdentifier()
// ---------------------------------------------------------------------
{
	$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$moc_identifier = '';
	for ($i = 0; $i < 32; $i++) {
		$moc_identifier .= $characters[rand(0, strlen($characters)-1)];
	}

	return $moc_identifier;
}

// ---------------------------------------------------------------------
function createTableMocIdentifier($dbh)
// ---------------------------------------------------------------------
{
  $rc = OK;

  $sql = "CREATE TABLE IF NOT EXISTS moc_identifier (
	moc_identifier     TEXT
, PRIMARY KEY (moc_identifier)
)";

  try {
    $dbh->exec($sql);
  }
  catch (PDOException $e) {
    $rc = 907;
  }

  return $rc;
}

?>
